unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

const
  AM_ADPAINT = WM_USER + $42;

type
  TMainForm = class(TForm)
    Panel1: TPanel;
    DeviceList: TListBox;
    DetectButton: TButton;
    ADMethodRadios: TRadioGroup;
    Panel2: TPanel;
    StartButton: TButton;
    ADPaint: TPaintBox;
    StopButton: TButton;
    Label1: TLabel;
    LogMemo: TMemo;
    ScanTime: TTimer;
    PollTime: TTimer;
    Splitter1: TSplitter;
    procedure FormCreate(Sender: TObject);
    procedure DetectButtonClick(Sender: TObject);
    procedure StartButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure ScanTimeTimer(Sender: TObject);
    procedure ADPaintPaint(Sender: TObject);
    procedure PollTimeTimer(Sender: TObject);
  protected
    procedure CompleteStop;
    procedure PostPaint;
    procedure AMADPaint(var Msg: TMessage); message AM_ADPAINT;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  AIOUSB;

type
  TADBlock = array[0..14] of Word;
  PADBlock = ^TADBlock;

var
  DeviceIndex: LongWord;
  LastTot, LastWt: LongWord;
  ADMethod: Integer;
  AvSamp: Integer = -1;
  pABlock, pBBlock: PADBlock;
  bStopBulk: Boolean;

procedure Swap(var A, D: Pointer); overload;
asm
  mov ECX, [EAX]
  xchg ECX, [EDX]
  mov [EAX], ECX
end;

procedure ADCallback(pBuf: PWord; BufSize, Flags, Context: LongWord); cdecl; forward;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  DetectButton.Align := alBottom;
  StartButton.Align := alTop;
  StopButton.Align := alTop;
  DetectButton.Click;

  New(pABlock);
  New(pBBlock);
end;

procedure TMainForm.DetectButtonClick(Sender: TObject);
const
  HexChar: array[0..15] of Char = ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F');
  
var
  I, J: Integer;
  Mask, Status, PID, NomSize: LongWord;
  Nom, SNStr: String;
  SN: Int64;

begin
  DeviceList.Items.Clear;
  Mask := GetDevices;
  for I := 0 to 31 do if (Mask and (1 shl I)) <> 0 then begin
    NomSize := 256;
    SetString(Nom, nil, NomSize);
    Status := QueryDeviceInfo(I, @PID, @NomSize, PChar(Nom), nil, nil);
    if Status <> ERROR_SUCCESS then Continue;
    if (PID and $FEC0) <> $8040 then Continue;
    SetLength(Nom, NomSize);
    Nom := StringReplace(Nom, #0, '', [rfReplaceAll]);

    Status := GetDeviceSerialNumber(I, SN);
    if Status <> ERROR_SUCCESS then Continue;
    SNStr := '';
    for J := 1 to 16 do begin
      SNStr := SNStr + HexChar[SN and $F];
      SN := SN shr 4;
    end;

    DeviceList.Items.Add(IntToStr(I) + ': ' + Nom + '(' + IntToHex(PID, 4) + ') ' + SNStr);
  end;

  if DeviceList.Items.Count <> 0 then begin
    StartButton.Enabled := True;
    DeviceList.ItemIndex := 0;
  end
  else begin
    StartButton.Enabled := False;
  end;
end;

procedure TMainForm.StartButtonClick(Sender: TObject);
var
  I: Integer;
  Buf: String;
  Status, ConfigSize: LongWord;
  Config: array[0..20] of Byte;
  Hz: Double;

begin
  AvSamp := -1;
  LastTot := 0;
  LastWt := 0;

  I := DeviceList.ItemIndex;
  if I < 0 then begin
    Beep;
    Exit;
  end;
  Buf := DeviceList.Items[I];
  I := Pos(':', Buf);
  if I < 2 then begin
    Beep;
    Exit;
  end;
  DeviceIndex := StrToInt(Copy(Buf, 1, I - 1));

  ADMethod := ADMethodRadios.ItemIndex;
  if ADMethod < 0 then begin
    Beep;
    Exit;
  end;

  DetectButton.Enabled := False;
  StartButton.Enabled := False;
  StopButton.Enabled := False;
  ADMethodRadios.Enabled := False;

  case ADMethod of
    0: LogMemo.Lines.Add('Starting get scan burnin...');
    1: LogMemo.Lines.Add('Starting continuous burnin...');
    2: LogMemo.Lines.Add('Starting bulk block burnin...');
  end;

  ADC_SetCal(DeviceIndex, ':1TO1:');

  for I := 0 to High(Config) do Config[I] := 0;
  Config[$11] := $05;
  Config[$13] := 14;
  ConfigSize := SizeOf(Config);
  Status := ADC_SetConfig(DeviceIndex, @Config[0], ConfigSize);
  if Status <> ERROR_SUCCESS then begin
    LogMemo.Lines.Add('Error ' + IntToStr(Status) + ' from ADC_SetConfig.');
    CompleteStop;
    Exit;
  end;

  case ADMethod of
    0: begin
      ScanTime.Enabled := True;
    end;
    1: begin
      Hz := 10;
      Status := CTR_StartOutputFreq(DeviceIndex, 0, @Hz);
      if Status <> ERROR_SUCCESS then begin
        LogMemo.Lines.Add('Error ' + IntToStr(Status) + ' from CTR_StartOutputFreq.');
        CompleteStop;
        Exit;
      end;
      ADC_BulkMode(DeviceIndex, $1);
      Status := ADC_BulkContinuousCallbackStart(DeviceIndex, $200, 6, 0, @ADCallback);
      if Status <> ERROR_SUCCESS then begin
        LogMemo.Lines.Add('Error ' + IntToStr(Status) + ' from ADC_BulkContinuousCallbackStart.');
        ADC_BulkMode(DeviceIndex, $0);
        CompleteStop;
        Exit;
      end;
    end;
    2: begin
      Hz := 10;
      Status := CTR_StartOutputFreq(DeviceIndex, 0, @Hz);
      if Status <> ERROR_SUCCESS then begin
        LogMemo.Lines.Add('Error ' + IntToStr(Status) + ' from CTR_StartOutputFreq.');
        CompleteStop;
        Exit;
      end;
      bStopBulk := False;
      Status := ADC_BulkAcquire(DeviceIndex, Length(pABlock^) * SizeOf(pABlock^[0]), @pABlock^[0]);
      if Status <> ERROR_SUCCESS then begin
        LogMemo.Lines.Add('Error ' + IntToStr(Status) + ' from ADC_BulkAcquire.');
        CompleteStop;
        Exit;
      end;
      PollTime.Enabled := True;
    end;
  end;

  StopButton.Enabled := True;
  LogMemo.Lines.Add('Started.');
end;

procedure TMainForm.StopButtonClick(Sender: TObject);
var
  Status, IOStatus: LongWord;

begin
  DetectButton.Enabled := False;
  StartButton.Enabled := False;
  StopButton.Enabled := False;
  ADMethodRadios.Enabled := False;

  case ADMethod of
    0: begin
      ScanTime.Enabled := False;
      CompleteStop;
    end;
    1: begin
      Status := ADC_BulkContinuousEnd(DeviceIndex, @IOStatus);
      if Status <> ERROR_SUCCESS then
        LogMemo.Lines.Add('Error ' + IntToStr(Status) + ' from ADC_BulkContinuousEnd.')
      else if IOStatus <> ERROR_SUCCESS then
        LogMemo.Lines.Add('Continuous ended(but I/O error ' + IntToStr(Status) + ' had occurred).')
      ;
      ADC_BulkMode(DeviceIndex, $0);
      CompleteStop;
    end;
    2: begin
      bStopBulk := True;
      //When the bulk is in, it will complete the stop.
    end;
  end;
end;

procedure TMainForm.CompleteStop;
var
  Hz: Double;

begin
  //AvSamp := -1;
  //PostPaint;

  Hz := 0;
  CTR_StartOutputFreq(DeviceIndex, 0, @Hz);

  DetectButton.Enabled := True;
  StartButton.Enabled := True;
  ADMethodRadios.Enabled := True;

  LogMemo.Lines.Add('Stopped.');
end;

procedure TMainForm.ScanTimeTimer(Sender: TObject);
var
  ADData: array of Word;

begin
  if not ScanTime.Enabled then Exit;

  SetLength(ADData, 128);
  ADC_GetScan(DeviceIndex, @ADData[0]);
  AvSamp := ADData[0];
  PostPaint;
  SetLength(ADData, 0);
end;

procedure ADCallback(pBuf: PWord; BufSize, Flags, Context: LongWord); cdecl;
var
  I: Integer;
  pSample: PWord;

begin
  pSample := pBuf;
  for I := 0 to (BufSize div 2) - 1 do begin
    Inc(LastTot, pSample^);
    Inc(LastWt);

    if LastWt >= 15 then begin
      AvSamp := Round(LastTot / LastWt);
      MainForm.PostPaint;
      LastTot := 0;
      LastWt := 0;
    end;
  end;
end;

procedure TMainForm.PollTimeTimer(Sender: TObject);
var
  Status, BytesLeft, Tot: LongWord;
  ThisBlock: PADBlock;
  I: Integer;

begin
  if not PollTime.Enabled then Exit;

  ThisBlock := pABlock;

  Status := ADC_BulkPoll(DeviceIndex, BytesLeft);
  if Status <> ERROR_SUCCESS then begin
    LogMemo.Lines.Add('Error ' + IntToStr(Status) + ' from ADC_BulkPoll.');
    PollTime.Enabled := False;
    CompleteStop;
    Exit;
  end;

  if BytesLeft <> 0 then Exit;

  if bStopBulk then begin
    PollTime.Enabled := False;
    CompleteStop;
  end
  else begin
    //Start the next one, on the other ping-ponging buffer.
    Swap(Pointer(pABlock), Pointer(pBBlock));
    Status := ADC_BulkAcquire(DeviceIndex, Length(pABlock^) * SizeOf(pABlock^[0]), @pABlock^[0]);
    if Status <> ERROR_SUCCESS then begin
      LogMemo.Lines.Add('Error ' + IntToStr(Status) + ' from ADC_BulkAcquire.');
      PollTime.Enabled := False;
      CompleteStop;
    end;
  end;

  Tot := 0;
  for I := 0 to High(ThisBlock^) do
    Inc(Tot, ThisBlock^[I])
  ;
  AvSamp := Round(Tot / Length(ThisBlock^));
  PostPaint;
end;

procedure TMainForm.PostPaint;
begin
  PostMessage(Handle, AM_ADPAINT, 0, 0);
end;

procedure TMainForm.AMADPaint(var Msg: TMessage);
begin
  ADPaint.Invalidate;
end;

procedure TMainForm.ADPaintPaint(Sender: TObject);
var
  ForeRect, BackRect: TRect;
  dY: Double;

begin
  if AvSamp < 0 then Exit;

  BackRect := ADPaint.ClientRect;
  with BackRect do begin
    Inc(Left, 8);
    Inc(Top, 8);
    Dec(Right, 8);
    Dec(Bottom, 8);
  end;
  ForeRect := BackRect;
  with ForeRect do begin
    dY := 1/$FFFF * (Bottom - Top);
    Top := Round(Bottom - dY * AvSamp);
  end;
  BackRect.Bottom := ForeRect.Top;
  ADPaint.Canvas.Brush.Color := $000000;
  ADPaint.Canvas.FillRect(BackRect);
  ADPaint.Canvas.Brush.Color := $FF0000;
  ADPaint.Canvas.FillRect(ForeRect);
end;

end.
